@@ -1,3 +1,30 @@
+make_find_root_file <- function(criterion) {
+  force(criterion)
+  eval(bquote(function(..., path = ".") {
+    find_root_file(..., criterion = criterion, path = path)
+  }))
+}
+
+make_fix_root_file <- function(criterion, path, subdir = NULL) {
+  root <- find_root(criterion = criterion, path = path)
+  if (!is.null(subdir)) {
+    root <- file.path(root, subdir)
+  }
+  eval(bquote(function(...) {
+    if (!missing(..1)) {
+      abs <- is_absolute_path(..1)
+      if (all(abs)) {
+        return(path(...))
+      }
+      if (any(abs)) {
+        stop("Combination of absolute and relative paths not supported.", call. = FALSE)
+      }
+    }
+
+    path(.(root), ...)
+  }))
+}
+
 #' Is a directory the project root?
 #'
 #' Objects of the `root_criterion` class decide if a
@@ -6,17 +33,19 @@
 #' Construct criteria using `root_criterion` in a very general fashion
 #' by specifying a function with a `path` argument, and a description.
 #'
-#' @param testfun A function with one parameter that returns `TRUE`
+#' @param testfun `[function|list(function)]`\cr
+#'   A function with one parameter that returns `TRUE`
 #'   if the directory specified by this parameter is the project root,
 #'   and `FALSE` otherwise. Can also be a list of such functions.
-#' @param desc A textual description of the test criterion, of the same length
-#'   as `testfun`
-#' @param subdir Subdirectories to start the search in, if found
+#' @param desc `[character]`\cr
+#'   A textual description of the test criterion, of the same length
+#'   as `testfun`.
+#' @param subdir `[character]`\cr
+#'   Subdirectories to start the search in, if found
 #'
 #' @return
 #' An S3 object of class `root_criterion` wit the following members:
 #'
-#' @include rrmake.R
 #' @export
 #'
 #' @examples
@@ -34,11 +63,13 @@ root_criterion <- function(testfun, desc, subdir = NULL) {
 
   full_desc <- paste0(
     desc,
-    if (!is.null(subdir)) paste0(
-      " (also look in subdirectories: ",
-      paste0("`", subdir, "`", collapse = ", "),
-      ")"
-    )
+    if (!is.null(subdir)) {
+      paste0(
+        " (also look in subdirectories: ",
+        paste0("`", subdir, "`", collapse = ", "),
+        ")"
+      )
+    }
   )
 
   criterion <- structure(
@@ -55,22 +86,28 @@ root_criterion <- function(testfun, desc, subdir = NULL) {
     class = "root_criterion"
   )
 
-  #'   \item{`find_file`}{A function with `...` argument that returns
-  #'     for a path relative to the root specified by this criterion.
+  #'   \item{`find_file`}{A function with `...` and `path` arguments
+  #'     that returns a path relative to the root,
+  #'     as specified by this criterion.
   #'     The optional `path` argument specifies the starting directory,
   #'     which defaults to `"."`.
+  #'     The function forwards to [find_root_file()],
+  #'     which passes `...` directly to `file.path()`
+  #'     if the first argument is an absolute path.
   #'   }
   criterion$find_file <- make_find_root_file(criterion)
   #'   \item{`make_fix_file`}{A function with a `path` argument that
   #'      returns a function that finds paths relative to the root.  For a
   #'      criterion `cr`, the result of `cr$make_fix_file(".")(...)`
   #'      is identical to `cr$find_file(...)`. The function created by
-  #'      `make_fix_file` can be saved to a variable to be more independent
+  #'      `make_fix_file()` can be saved to a variable to be more independent
   #'      of the current working directory.
   #'   }
   #' }
   criterion$make_fix_file <-
-    function(path = getwd()) make_fix_root_file(criterion, path)
+    function(path = getwd(), subdir = NULL) {
+      make_fix_root_file(criterion, path, subdir)
+    }
 
   criterion
 }
@@ -82,70 +119,9 @@ check_testfun <- function(testfun) {
 
   for (f in testfun) {
     if (!isTRUE(all.equal(names(formals(f)), "path"))) {
-      stop("All functions in testfun must have exactly one argument 'path'")
+      stop("All functions in testfun must have exactly one argument 'path'", call. = FALSE)
     }
   }
 
   testfun
 }
-
-#' @rdname root_criterion
-#' @param x An object
-#' @export
-is.root_criterion <- function(x) {
-  inherits(x, "root_criterion")
-}
-
-#' @rdname root_criterion
-#' @export
-as.root_criterion <- function(x) UseMethod("as.root_criterion", x)
-
-#' @details
-#' The `as.root_criterion()` function accepts objects of class
-#' `root_criterion`, and character values; the latter will be
-#' converted to criteria using `has_file`.
-#'
-#' @rdname root_criterion
-#' @export
-as.root_criterion.character <- function(x) {
-  has_file(x)
-}
-
-#' @rdname root_criterion
-#' @export
-as.root_criterion.root_criterion <- identity
-
-#' @export
-as.root_criterion.default <- function(x) {
-  stop("Cannot coerce ", x, " to type root_criterion.")
-}
-
-#' @export
-format.root_criterion <- function(x, ...) {
-  if (length(x$desc) > 1) {
-    c("Root criterion: one of", paste0("- ", x$desc))
-  } else {
-    paste0("Root criterion: ", x$desc)
-  }
-}
-
-#' @export
-print.root_criterion <- function(x, ...) {
-  cat(format(x), sep = "\n")
-  invisible(x)
-}
-
-#' @export
-#' @rdname root_criterion
-#' @details Root criteria can be combined with the `|` operator. The result is a
-#'   composite root criterion that requires either of the original criteria to
-#'   match.
-#' @param y An object
-`|.root_criterion` <- function(x, y) {
-  stopifnot(is.root_criterion(y))
-
-  root_criterion(
-    c(x$testfun, y$testfun),
-    c(x$desc, y$desc)
-  )
-}
